# This file was auto-generated by Fern from our API Definition.

import typing
import urllib.parse
from json.decoder import JSONDecodeError

import pydantic

from .....core.api_error import ApiError
from .....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .....core.jsonable_encoder import jsonable_encoder
from .....core.remove_none_from_dict import remove_none_from_dict
from ...errors.bad_request_error import BadRequestError
from ...errors.forbidden_error import ForbiddenError
from ...errors.internal_server_error import InternalServerError
from ...errors.not_implemented_error import NotImplementedError
from ...errors.too_many_requests_error import TooManyRequestsError
from ...types.create_employee_result import CreateEmployeeResult
from ...types.employee_result import EmployeeResult
from ...types.employees_paginated import EmployeesPaginated
from ...types.hris_create_employee_request_dto import HrisCreateEmployeeRequestDto

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class EmployeesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def hris_employees_list(
        self,
        *,
        page: typing.Optional[str] = None,
        page_size: typing.Optional[str] = None,
        raw: typing.Optional[bool] = None,
        fields: typing.Optional[str] = None,
        sync_token: typing.Optional[str] = None,
        updated_after: typing.Optional[str] = None,
        expand: typing.Optional[str] = None,
        account_id: str,
    ) -> EmployeesPaginated:
        """
        Parameters:
            - page: typing.Optional[str]. The page number of the results to fetch

            - page_size: typing.Optional[str]. The number of results per page

            - raw: typing.Optional[bool]. Indicates that the raw request result is returned

            - fields: typing.Optional[str]. The comma separated list of fields to return in the response (if empty, all fields are returned)

            - sync_token: typing.Optional[str]. The sync token to select the only updated results

            - updated_after: typing.Optional[str]. Use a string with a date to only select results updated after that given date

            - expand: typing.Optional[str]. The comma separated list of fields that will be expanded in the response

            - account_id: str. The account identifier
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "unified/hris/employees"),
            params=remove_none_from_dict(
                {
                    "page": page,
                    "page_size": page_size,
                    "raw": raw,
                    "fields": fields,
                    "sync_token": sync_token,
                    "updated_after": updated_after,
                    "expand": expand,
                }
            ),
            headers=remove_none_from_dict({**self._client_wrapper.get_headers(), "x-account-id": account_id}),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(EmployeesPaginated, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 501:
            raise NotImplementedError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def employee_create(self, *, request: HrisCreateEmployeeRequestDto, account_id: str) -> CreateEmployeeResult:
        """
        Parameters:
            - request: HrisCreateEmployeeRequestDto.

            - account_id: str. The account identifier
        """
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "unified/hris/employees"),
            json=jsonable_encoder(request),
            headers=remove_none_from_dict({**self._client_wrapper.get_headers(), "x-account-id": account_id}),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(CreateEmployeeResult, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 501:
            raise NotImplementedError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def hris_employees_get(
        self,
        id: str,
        *,
        page: typing.Optional[str] = None,
        page_size: typing.Optional[str] = None,
        raw: typing.Optional[bool] = None,
        fields: typing.Optional[str] = None,
        sync_token: typing.Optional[str] = None,
        updated_after: typing.Optional[str] = None,
        expand: typing.Optional[str] = None,
        account_id: str,
    ) -> EmployeeResult:
        """
        Parameters:
            - id: str.

            - page: typing.Optional[str]. The page number of the results to fetch

            - page_size: typing.Optional[str]. The number of results per page

            - raw: typing.Optional[bool]. Indicates that the raw request result is returned

            - fields: typing.Optional[str]. The comma separated list of fields to return in the response (if empty, all fields are returned)

            - sync_token: typing.Optional[str]. The sync token to select the only updated results

            - updated_after: typing.Optional[str]. Use a string with a date to only select results updated after that given date

            - expand: typing.Optional[str]. The comma separated list of fields that will be expanded in the response

            - account_id: str. The account identifier
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"unified/hris/employees/{id}"),
            params=remove_none_from_dict(
                {
                    "page": page,
                    "page_size": page_size,
                    "raw": raw,
                    "fields": fields,
                    "sync_token": sync_token,
                    "updated_after": updated_after,
                    "expand": expand,
                }
            ),
            headers=remove_none_from_dict({**self._client_wrapper.get_headers(), "x-account-id": account_id}),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(EmployeeResult, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 501:
            raise NotImplementedError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def employee_replace(
        self, id: str, *, request: HrisCreateEmployeeRequestDto, account_id: str
    ) -> CreateEmployeeResult:
        """
        Parameters:
            - id: str.

            - request: HrisCreateEmployeeRequestDto.

            - account_id: str. The account identifier
        """
        _response = self._client_wrapper.httpx_client.request(
            "PUT",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"unified/hris/employees/{id}"),
            json=jsonable_encoder(request),
            headers=remove_none_from_dict({**self._client_wrapper.get_headers(), "x-account-id": account_id}),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(CreateEmployeeResult, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 501:
            raise NotImplementedError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def employee_update(
        self, id: str, *, request: HrisCreateEmployeeRequestDto, account_id: str
    ) -> CreateEmployeeResult:
        """
        Parameters:
            - id: str.

            - request: HrisCreateEmployeeRequestDto.

            - account_id: str. The account identifier
        """
        _response = self._client_wrapper.httpx_client.request(
            "PATCH",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"unified/hris/employees/{id}"),
            json=jsonable_encoder(request),
            headers=remove_none_from_dict({**self._client_wrapper.get_headers(), "x-account-id": account_id}),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(CreateEmployeeResult, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 501:
            raise NotImplementedError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncEmployeesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def hris_employees_list(
        self,
        *,
        page: typing.Optional[str] = None,
        page_size: typing.Optional[str] = None,
        raw: typing.Optional[bool] = None,
        fields: typing.Optional[str] = None,
        sync_token: typing.Optional[str] = None,
        updated_after: typing.Optional[str] = None,
        expand: typing.Optional[str] = None,
        account_id: str,
    ) -> EmployeesPaginated:
        """
        Parameters:
            - page: typing.Optional[str]. The page number of the results to fetch

            - page_size: typing.Optional[str]. The number of results per page

            - raw: typing.Optional[bool]. Indicates that the raw request result is returned

            - fields: typing.Optional[str]. The comma separated list of fields to return in the response (if empty, all fields are returned)

            - sync_token: typing.Optional[str]. The sync token to select the only updated results

            - updated_after: typing.Optional[str]. Use a string with a date to only select results updated after that given date

            - expand: typing.Optional[str]. The comma separated list of fields that will be expanded in the response

            - account_id: str. The account identifier
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "unified/hris/employees"),
            params=remove_none_from_dict(
                {
                    "page": page,
                    "page_size": page_size,
                    "raw": raw,
                    "fields": fields,
                    "sync_token": sync_token,
                    "updated_after": updated_after,
                    "expand": expand,
                }
            ),
            headers=remove_none_from_dict({**self._client_wrapper.get_headers(), "x-account-id": account_id}),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(EmployeesPaginated, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 501:
            raise NotImplementedError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def employee_create(self, *, request: HrisCreateEmployeeRequestDto, account_id: str) -> CreateEmployeeResult:
        """
        Parameters:
            - request: HrisCreateEmployeeRequestDto.

            - account_id: str. The account identifier
        """
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "unified/hris/employees"),
            json=jsonable_encoder(request),
            headers=remove_none_from_dict({**self._client_wrapper.get_headers(), "x-account-id": account_id}),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(CreateEmployeeResult, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 501:
            raise NotImplementedError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def hris_employees_get(
        self,
        id: str,
        *,
        page: typing.Optional[str] = None,
        page_size: typing.Optional[str] = None,
        raw: typing.Optional[bool] = None,
        fields: typing.Optional[str] = None,
        sync_token: typing.Optional[str] = None,
        updated_after: typing.Optional[str] = None,
        expand: typing.Optional[str] = None,
        account_id: str,
    ) -> EmployeeResult:
        """
        Parameters:
            - id: str.

            - page: typing.Optional[str]. The page number of the results to fetch

            - page_size: typing.Optional[str]. The number of results per page

            - raw: typing.Optional[bool]. Indicates that the raw request result is returned

            - fields: typing.Optional[str]. The comma separated list of fields to return in the response (if empty, all fields are returned)

            - sync_token: typing.Optional[str]. The sync token to select the only updated results

            - updated_after: typing.Optional[str]. Use a string with a date to only select results updated after that given date

            - expand: typing.Optional[str]. The comma separated list of fields that will be expanded in the response

            - account_id: str. The account identifier
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"unified/hris/employees/{id}"),
            params=remove_none_from_dict(
                {
                    "page": page,
                    "page_size": page_size,
                    "raw": raw,
                    "fields": fields,
                    "sync_token": sync_token,
                    "updated_after": updated_after,
                    "expand": expand,
                }
            ),
            headers=remove_none_from_dict({**self._client_wrapper.get_headers(), "x-account-id": account_id}),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(EmployeeResult, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 501:
            raise NotImplementedError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def employee_replace(
        self, id: str, *, request: HrisCreateEmployeeRequestDto, account_id: str
    ) -> CreateEmployeeResult:
        """
        Parameters:
            - id: str.

            - request: HrisCreateEmployeeRequestDto.

            - account_id: str. The account identifier
        """
        _response = await self._client_wrapper.httpx_client.request(
            "PUT",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"unified/hris/employees/{id}"),
            json=jsonable_encoder(request),
            headers=remove_none_from_dict({**self._client_wrapper.get_headers(), "x-account-id": account_id}),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(CreateEmployeeResult, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 501:
            raise NotImplementedError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def employee_update(
        self, id: str, *, request: HrisCreateEmployeeRequestDto, account_id: str
    ) -> CreateEmployeeResult:
        """
        Parameters:
            - id: str.

            - request: HrisCreateEmployeeRequestDto.

            - account_id: str. The account identifier
        """
        _response = await self._client_wrapper.httpx_client.request(
            "PATCH",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"unified/hris/employees/{id}"),
            json=jsonable_encoder(request),
            headers=remove_none_from_dict({**self._client_wrapper.get_headers(), "x-account-id": account_id}),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(CreateEmployeeResult, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 501:
            raise NotImplementedError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)
