# This file was auto-generated by Fern from our API Definition.

import typing
import urllib.parse
from json.decoder import JSONDecodeError

import pydantic

from .....core.api_error import ApiError
from .....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .....core.jsonable_encoder import jsonable_encoder
from .....core.remove_none_from_dict import remove_none_from_dict
from ...errors.bad_request_error import BadRequestError
from ...errors.forbidden_error import ForbiddenError
from ...errors.internal_server_error import InternalServerError
from ...errors.not_implemented_error import NotImplementedError
from ...errors.too_many_requests_error import TooManyRequestsError
from ...errors.unprocessable_entity_error import UnprocessableEntityError
from ...types.create_template_result import CreateTemplateResult
from ...types.marketing_create_email_template_request_dto import MarketingCreateEmailTemplateRequestDto
from ...types.marketing_create_push_template_request_dto import MarketingCreatePushTemplateRequestDto
from ...types.marketing_create_template_request_dto import MarketingCreateTemplateRequestDto
from ...types.template_result import TemplateResult
from ...types.templates_paginated import TemplatesPaginated

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class TemplatesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def marketing_templates_list(self, *, account_id: str) -> None:
        """
        Parameters:
            - account_id: str. The account identifier
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "unified/marketing/templates"),
            headers=remove_none_from_dict({**self._client_wrapper.get_headers(), "x-account-id": account_id}),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 501:
            raise NotImplementedError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def marketing_email_templates_list(
        self,
        *,
        page: typing.Optional[str] = None,
        page_size: typing.Optional[str] = None,
        raw: typing.Optional[bool] = None,
        fields: typing.Optional[str] = None,
        sync_token: typing.Optional[str] = None,
        updated_after: typing.Optional[str] = None,
        account_id: str,
    ) -> TemplatesPaginated:
        """
        Parameters:
            - page: typing.Optional[str]. The page number of the results to fetch

            - page_size: typing.Optional[str]. The number of results per page

            - raw: typing.Optional[bool]. Indicates that the raw request result is returned

            - fields: typing.Optional[str]. The comma separated list of fields to return in the response (if empty, all fields are returned)

            - sync_token: typing.Optional[str]. The sync token to select the only updated results

            - updated_after: typing.Optional[str]. Use a string with a date to only select results updated after that given date

            - account_id: str. The account identifier
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "unified/marketing/templates/email"),
            params=remove_none_from_dict(
                {
                    "page": page,
                    "page_size": page_size,
                    "raw": raw,
                    "fields": fields,
                    "sync_token": sync_token,
                    "updated_after": updated_after,
                }
            ),
            headers=remove_none_from_dict({**self._client_wrapper.get_headers(), "x-account-id": account_id}),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(TemplatesPaginated, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 501:
            raise NotImplementedError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def marketing_email_template_create(
        self, *, request: MarketingCreateEmailTemplateRequestDto, account_id: str
    ) -> CreateTemplateResult:
        """
        Parameters:
            - request: MarketingCreateEmailTemplateRequestDto.

            - account_id: str. The account identifier
        """
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "unified/marketing/templates/email"),
            json=jsonable_encoder(request),
            headers=remove_none_from_dict({**self._client_wrapper.get_headers(), "x-account-id": account_id}),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(CreateTemplateResult, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 501:
            raise NotImplementedError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def marketing_email_templates_get(
        self,
        id: str,
        *,
        page: typing.Optional[str] = None,
        page_size: typing.Optional[str] = None,
        raw: typing.Optional[bool] = None,
        fields: typing.Optional[str] = None,
        sync_token: typing.Optional[str] = None,
        updated_after: typing.Optional[str] = None,
        account_id: str,
    ) -> TemplateResult:
        """
        Parameters:
            - id: str.

            - page: typing.Optional[str]. The page number of the results to fetch

            - page_size: typing.Optional[str]. The number of results per page

            - raw: typing.Optional[bool]. Indicates that the raw request result is returned

            - fields: typing.Optional[str]. The comma separated list of fields to return in the response (if empty, all fields are returned)

            - sync_token: typing.Optional[str]. The sync token to select the only updated results

            - updated_after: typing.Optional[str]. Use a string with a date to only select results updated after that given date

            - account_id: str. The account identifier
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"unified/marketing/templates/email/{id}"),
            params=remove_none_from_dict(
                {
                    "page": page,
                    "page_size": page_size,
                    "raw": raw,
                    "fields": fields,
                    "sync_token": sync_token,
                    "updated_after": updated_after,
                }
            ),
            headers=remove_none_from_dict({**self._client_wrapper.get_headers(), "x-account-id": account_id}),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(TemplateResult, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 501:
            raise NotImplementedError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def marketing_email_template_update(
        self, id: str, *, request: MarketingCreateEmailTemplateRequestDto, account_id: str
    ) -> CreateTemplateResult:
        """
        Parameters:
            - id: str.

            - request: MarketingCreateEmailTemplateRequestDto.

            - account_id: str. The account identifier
        """
        _response = self._client_wrapper.httpx_client.request(
            "PATCH",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"unified/marketing/templates/email/{id}"),
            json=jsonable_encoder(request),
            headers=remove_none_from_dict({**self._client_wrapper.get_headers(), "x-account-id": account_id}),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(CreateTemplateResult, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 501:
            raise NotImplementedError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def marketing_omni_channel_templates_list(
        self,
        *,
        page: typing.Optional[str] = None,
        page_size: typing.Optional[str] = None,
        raw: typing.Optional[bool] = None,
        fields: typing.Optional[str] = None,
        sync_token: typing.Optional[str] = None,
        updated_after: typing.Optional[str] = None,
        account_id: str,
    ) -> TemplatesPaginated:
        """
        Parameters:
            - page: typing.Optional[str]. The page number of the results to fetch

            - page_size: typing.Optional[str]. The number of results per page

            - raw: typing.Optional[bool]. Indicates that the raw request result is returned

            - fields: typing.Optional[str]. The comma separated list of fields to return in the response (if empty, all fields are returned)

            - sync_token: typing.Optional[str]. The sync token to select the only updated results

            - updated_after: typing.Optional[str]. Use a string with a date to only select results updated after that given date

            - account_id: str. The account identifier
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "unified/marketing/templates/omni_channel"),
            params=remove_none_from_dict(
                {
                    "page": page,
                    "page_size": page_size,
                    "raw": raw,
                    "fields": fields,
                    "sync_token": sync_token,
                    "updated_after": updated_after,
                }
            ),
            headers=remove_none_from_dict({**self._client_wrapper.get_headers(), "x-account-id": account_id}),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(TemplatesPaginated, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 501:
            raise NotImplementedError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def marketing_omni_channel_template_create(
        self, *, request: MarketingCreateTemplateRequestDto, account_id: str
    ) -> CreateTemplateResult:
        """
        Parameters:
            - request: MarketingCreateTemplateRequestDto.

            - account_id: str. The account identifier
        """
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "unified/marketing/templates/omni_channel"),
            json=jsonable_encoder(request),
            headers=remove_none_from_dict({**self._client_wrapper.get_headers(), "x-account-id": account_id}),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(CreateTemplateResult, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 501:
            raise NotImplementedError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def marketing_omni_channel_templates_get(
        self,
        id: str,
        *,
        page: typing.Optional[str] = None,
        page_size: typing.Optional[str] = None,
        raw: typing.Optional[bool] = None,
        fields: typing.Optional[str] = None,
        sync_token: typing.Optional[str] = None,
        updated_after: typing.Optional[str] = None,
        account_id: str,
    ) -> TemplateResult:
        """
        Parameters:
            - id: str.

            - page: typing.Optional[str]. The page number of the results to fetch

            - page_size: typing.Optional[str]. The number of results per page

            - raw: typing.Optional[bool]. Indicates that the raw request result is returned

            - fields: typing.Optional[str]. The comma separated list of fields to return in the response (if empty, all fields are returned)

            - sync_token: typing.Optional[str]. The sync token to select the only updated results

            - updated_after: typing.Optional[str]. Use a string with a date to only select results updated after that given date

            - account_id: str. The account identifier
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"unified/marketing/templates/omni_channel/{id}"
            ),
            params=remove_none_from_dict(
                {
                    "page": page,
                    "page_size": page_size,
                    "raw": raw,
                    "fields": fields,
                    "sync_token": sync_token,
                    "updated_after": updated_after,
                }
            ),
            headers=remove_none_from_dict({**self._client_wrapper.get_headers(), "x-account-id": account_id}),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(TemplateResult, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 501:
            raise NotImplementedError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def marketing_omni_channel_template_update(
        self, id: str, *, request: MarketingCreateTemplateRequestDto, account_id: str
    ) -> CreateTemplateResult:
        """
        Parameters:
            - id: str.

            - request: MarketingCreateTemplateRequestDto.

            - account_id: str. The account identifier
        """
        _response = self._client_wrapper.httpx_client.request(
            "PATCH",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"unified/marketing/templates/omni_channel/{id}"
            ),
            json=jsonable_encoder(request),
            headers=remove_none_from_dict({**self._client_wrapper.get_headers(), "x-account-id": account_id}),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(CreateTemplateResult, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 501:
            raise NotImplementedError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def marketing_push_templates_list(
        self,
        *,
        page: typing.Optional[str] = None,
        page_size: typing.Optional[str] = None,
        raw: typing.Optional[bool] = None,
        fields: typing.Optional[str] = None,
        sync_token: typing.Optional[str] = None,
        updated_after: typing.Optional[str] = None,
        account_id: str,
    ) -> TemplatesPaginated:
        """
        Parameters:
            - page: typing.Optional[str]. The page number of the results to fetch

            - page_size: typing.Optional[str]. The number of results per page

            - raw: typing.Optional[bool]. Indicates that the raw request result is returned

            - fields: typing.Optional[str]. The comma separated list of fields to return in the response (if empty, all fields are returned)

            - sync_token: typing.Optional[str]. The sync token to select the only updated results

            - updated_after: typing.Optional[str]. Use a string with a date to only select results updated after that given date

            - account_id: str. The account identifier
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "unified/marketing/templates/push"),
            params=remove_none_from_dict(
                {
                    "page": page,
                    "page_size": page_size,
                    "raw": raw,
                    "fields": fields,
                    "sync_token": sync_token,
                    "updated_after": updated_after,
                }
            ),
            headers=remove_none_from_dict({**self._client_wrapper.get_headers(), "x-account-id": account_id}),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(TemplatesPaginated, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 501:
            raise NotImplementedError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def marketing_push_template_create(
        self, *, request: MarketingCreatePushTemplateRequestDto, account_id: str
    ) -> CreateTemplateResult:
        """
        Parameters:
            - request: MarketingCreatePushTemplateRequestDto.

            - account_id: str. The account identifier
        """
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "unified/marketing/templates/push"),
            json=jsonable_encoder(request),
            headers=remove_none_from_dict({**self._client_wrapper.get_headers(), "x-account-id": account_id}),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(CreateTemplateResult, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 501:
            raise NotImplementedError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def marketing_push_templates_get(
        self,
        id: str,
        *,
        page: typing.Optional[str] = None,
        page_size: typing.Optional[str] = None,
        raw: typing.Optional[bool] = None,
        fields: typing.Optional[str] = None,
        sync_token: typing.Optional[str] = None,
        updated_after: typing.Optional[str] = None,
        account_id: str,
    ) -> TemplateResult:
        """
        Parameters:
            - id: str.

            - page: typing.Optional[str]. The page number of the results to fetch

            - page_size: typing.Optional[str]. The number of results per page

            - raw: typing.Optional[bool]. Indicates that the raw request result is returned

            - fields: typing.Optional[str]. The comma separated list of fields to return in the response (if empty, all fields are returned)

            - sync_token: typing.Optional[str]. The sync token to select the only updated results

            - updated_after: typing.Optional[str]. Use a string with a date to only select results updated after that given date

            - account_id: str. The account identifier
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"unified/marketing/templates/push/{id}"),
            params=remove_none_from_dict(
                {
                    "page": page,
                    "page_size": page_size,
                    "raw": raw,
                    "fields": fields,
                    "sync_token": sync_token,
                    "updated_after": updated_after,
                }
            ),
            headers=remove_none_from_dict({**self._client_wrapper.get_headers(), "x-account-id": account_id}),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(TemplateResult, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 501:
            raise NotImplementedError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def marketing_push_template_update(
        self, id: str, *, request: MarketingCreatePushTemplateRequestDto, account_id: str
    ) -> CreateTemplateResult:
        """
        Parameters:
            - id: str.

            - request: MarketingCreatePushTemplateRequestDto.

            - account_id: str. The account identifier
        """
        _response = self._client_wrapper.httpx_client.request(
            "PATCH",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"unified/marketing/templates/push/{id}"),
            json=jsonable_encoder(request),
            headers=remove_none_from_dict({**self._client_wrapper.get_headers(), "x-account-id": account_id}),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(CreateTemplateResult, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 501:
            raise NotImplementedError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncTemplatesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def marketing_templates_list(self, *, account_id: str) -> None:
        """
        Parameters:
            - account_id: str. The account identifier
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "unified/marketing/templates"),
            headers=remove_none_from_dict({**self._client_wrapper.get_headers(), "x-account-id": account_id}),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 501:
            raise NotImplementedError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def marketing_email_templates_list(
        self,
        *,
        page: typing.Optional[str] = None,
        page_size: typing.Optional[str] = None,
        raw: typing.Optional[bool] = None,
        fields: typing.Optional[str] = None,
        sync_token: typing.Optional[str] = None,
        updated_after: typing.Optional[str] = None,
        account_id: str,
    ) -> TemplatesPaginated:
        """
        Parameters:
            - page: typing.Optional[str]. The page number of the results to fetch

            - page_size: typing.Optional[str]. The number of results per page

            - raw: typing.Optional[bool]. Indicates that the raw request result is returned

            - fields: typing.Optional[str]. The comma separated list of fields to return in the response (if empty, all fields are returned)

            - sync_token: typing.Optional[str]. The sync token to select the only updated results

            - updated_after: typing.Optional[str]. Use a string with a date to only select results updated after that given date

            - account_id: str. The account identifier
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "unified/marketing/templates/email"),
            params=remove_none_from_dict(
                {
                    "page": page,
                    "page_size": page_size,
                    "raw": raw,
                    "fields": fields,
                    "sync_token": sync_token,
                    "updated_after": updated_after,
                }
            ),
            headers=remove_none_from_dict({**self._client_wrapper.get_headers(), "x-account-id": account_id}),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(TemplatesPaginated, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 501:
            raise NotImplementedError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def marketing_email_template_create(
        self, *, request: MarketingCreateEmailTemplateRequestDto, account_id: str
    ) -> CreateTemplateResult:
        """
        Parameters:
            - request: MarketingCreateEmailTemplateRequestDto.

            - account_id: str. The account identifier
        """
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "unified/marketing/templates/email"),
            json=jsonable_encoder(request),
            headers=remove_none_from_dict({**self._client_wrapper.get_headers(), "x-account-id": account_id}),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(CreateTemplateResult, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 501:
            raise NotImplementedError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def marketing_email_templates_get(
        self,
        id: str,
        *,
        page: typing.Optional[str] = None,
        page_size: typing.Optional[str] = None,
        raw: typing.Optional[bool] = None,
        fields: typing.Optional[str] = None,
        sync_token: typing.Optional[str] = None,
        updated_after: typing.Optional[str] = None,
        account_id: str,
    ) -> TemplateResult:
        """
        Parameters:
            - id: str.

            - page: typing.Optional[str]. The page number of the results to fetch

            - page_size: typing.Optional[str]. The number of results per page

            - raw: typing.Optional[bool]. Indicates that the raw request result is returned

            - fields: typing.Optional[str]. The comma separated list of fields to return in the response (if empty, all fields are returned)

            - sync_token: typing.Optional[str]. The sync token to select the only updated results

            - updated_after: typing.Optional[str]. Use a string with a date to only select results updated after that given date

            - account_id: str. The account identifier
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"unified/marketing/templates/email/{id}"),
            params=remove_none_from_dict(
                {
                    "page": page,
                    "page_size": page_size,
                    "raw": raw,
                    "fields": fields,
                    "sync_token": sync_token,
                    "updated_after": updated_after,
                }
            ),
            headers=remove_none_from_dict({**self._client_wrapper.get_headers(), "x-account-id": account_id}),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(TemplateResult, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 501:
            raise NotImplementedError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def marketing_email_template_update(
        self, id: str, *, request: MarketingCreateEmailTemplateRequestDto, account_id: str
    ) -> CreateTemplateResult:
        """
        Parameters:
            - id: str.

            - request: MarketingCreateEmailTemplateRequestDto.

            - account_id: str. The account identifier
        """
        _response = await self._client_wrapper.httpx_client.request(
            "PATCH",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"unified/marketing/templates/email/{id}"),
            json=jsonable_encoder(request),
            headers=remove_none_from_dict({**self._client_wrapper.get_headers(), "x-account-id": account_id}),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(CreateTemplateResult, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 501:
            raise NotImplementedError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def marketing_omni_channel_templates_list(
        self,
        *,
        page: typing.Optional[str] = None,
        page_size: typing.Optional[str] = None,
        raw: typing.Optional[bool] = None,
        fields: typing.Optional[str] = None,
        sync_token: typing.Optional[str] = None,
        updated_after: typing.Optional[str] = None,
        account_id: str,
    ) -> TemplatesPaginated:
        """
        Parameters:
            - page: typing.Optional[str]. The page number of the results to fetch

            - page_size: typing.Optional[str]. The number of results per page

            - raw: typing.Optional[bool]. Indicates that the raw request result is returned

            - fields: typing.Optional[str]. The comma separated list of fields to return in the response (if empty, all fields are returned)

            - sync_token: typing.Optional[str]. The sync token to select the only updated results

            - updated_after: typing.Optional[str]. Use a string with a date to only select results updated after that given date

            - account_id: str. The account identifier
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "unified/marketing/templates/omni_channel"),
            params=remove_none_from_dict(
                {
                    "page": page,
                    "page_size": page_size,
                    "raw": raw,
                    "fields": fields,
                    "sync_token": sync_token,
                    "updated_after": updated_after,
                }
            ),
            headers=remove_none_from_dict({**self._client_wrapper.get_headers(), "x-account-id": account_id}),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(TemplatesPaginated, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 501:
            raise NotImplementedError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def marketing_omni_channel_template_create(
        self, *, request: MarketingCreateTemplateRequestDto, account_id: str
    ) -> CreateTemplateResult:
        """
        Parameters:
            - request: MarketingCreateTemplateRequestDto.

            - account_id: str. The account identifier
        """
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "unified/marketing/templates/omni_channel"),
            json=jsonable_encoder(request),
            headers=remove_none_from_dict({**self._client_wrapper.get_headers(), "x-account-id": account_id}),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(CreateTemplateResult, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 501:
            raise NotImplementedError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def marketing_omni_channel_templates_get(
        self,
        id: str,
        *,
        page: typing.Optional[str] = None,
        page_size: typing.Optional[str] = None,
        raw: typing.Optional[bool] = None,
        fields: typing.Optional[str] = None,
        sync_token: typing.Optional[str] = None,
        updated_after: typing.Optional[str] = None,
        account_id: str,
    ) -> TemplateResult:
        """
        Parameters:
            - id: str.

            - page: typing.Optional[str]. The page number of the results to fetch

            - page_size: typing.Optional[str]. The number of results per page

            - raw: typing.Optional[bool]. Indicates that the raw request result is returned

            - fields: typing.Optional[str]. The comma separated list of fields to return in the response (if empty, all fields are returned)

            - sync_token: typing.Optional[str]. The sync token to select the only updated results

            - updated_after: typing.Optional[str]. Use a string with a date to only select results updated after that given date

            - account_id: str. The account identifier
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"unified/marketing/templates/omni_channel/{id}"
            ),
            params=remove_none_from_dict(
                {
                    "page": page,
                    "page_size": page_size,
                    "raw": raw,
                    "fields": fields,
                    "sync_token": sync_token,
                    "updated_after": updated_after,
                }
            ),
            headers=remove_none_from_dict({**self._client_wrapper.get_headers(), "x-account-id": account_id}),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(TemplateResult, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 501:
            raise NotImplementedError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def marketing_omni_channel_template_update(
        self, id: str, *, request: MarketingCreateTemplateRequestDto, account_id: str
    ) -> CreateTemplateResult:
        """
        Parameters:
            - id: str.

            - request: MarketingCreateTemplateRequestDto.

            - account_id: str. The account identifier
        """
        _response = await self._client_wrapper.httpx_client.request(
            "PATCH",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"unified/marketing/templates/omni_channel/{id}"
            ),
            json=jsonable_encoder(request),
            headers=remove_none_from_dict({**self._client_wrapper.get_headers(), "x-account-id": account_id}),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(CreateTemplateResult, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 501:
            raise NotImplementedError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def marketing_push_templates_list(
        self,
        *,
        page: typing.Optional[str] = None,
        page_size: typing.Optional[str] = None,
        raw: typing.Optional[bool] = None,
        fields: typing.Optional[str] = None,
        sync_token: typing.Optional[str] = None,
        updated_after: typing.Optional[str] = None,
        account_id: str,
    ) -> TemplatesPaginated:
        """
        Parameters:
            - page: typing.Optional[str]. The page number of the results to fetch

            - page_size: typing.Optional[str]. The number of results per page

            - raw: typing.Optional[bool]. Indicates that the raw request result is returned

            - fields: typing.Optional[str]. The comma separated list of fields to return in the response (if empty, all fields are returned)

            - sync_token: typing.Optional[str]. The sync token to select the only updated results

            - updated_after: typing.Optional[str]. Use a string with a date to only select results updated after that given date

            - account_id: str. The account identifier
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "unified/marketing/templates/push"),
            params=remove_none_from_dict(
                {
                    "page": page,
                    "page_size": page_size,
                    "raw": raw,
                    "fields": fields,
                    "sync_token": sync_token,
                    "updated_after": updated_after,
                }
            ),
            headers=remove_none_from_dict({**self._client_wrapper.get_headers(), "x-account-id": account_id}),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(TemplatesPaginated, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 501:
            raise NotImplementedError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def marketing_push_template_create(
        self, *, request: MarketingCreatePushTemplateRequestDto, account_id: str
    ) -> CreateTemplateResult:
        """
        Parameters:
            - request: MarketingCreatePushTemplateRequestDto.

            - account_id: str. The account identifier
        """
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "unified/marketing/templates/push"),
            json=jsonable_encoder(request),
            headers=remove_none_from_dict({**self._client_wrapper.get_headers(), "x-account-id": account_id}),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(CreateTemplateResult, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 501:
            raise NotImplementedError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def marketing_push_templates_get(
        self,
        id: str,
        *,
        page: typing.Optional[str] = None,
        page_size: typing.Optional[str] = None,
        raw: typing.Optional[bool] = None,
        fields: typing.Optional[str] = None,
        sync_token: typing.Optional[str] = None,
        updated_after: typing.Optional[str] = None,
        account_id: str,
    ) -> TemplateResult:
        """
        Parameters:
            - id: str.

            - page: typing.Optional[str]. The page number of the results to fetch

            - page_size: typing.Optional[str]. The number of results per page

            - raw: typing.Optional[bool]. Indicates that the raw request result is returned

            - fields: typing.Optional[str]. The comma separated list of fields to return in the response (if empty, all fields are returned)

            - sync_token: typing.Optional[str]. The sync token to select the only updated results

            - updated_after: typing.Optional[str]. Use a string with a date to only select results updated after that given date

            - account_id: str. The account identifier
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"unified/marketing/templates/push/{id}"),
            params=remove_none_from_dict(
                {
                    "page": page,
                    "page_size": page_size,
                    "raw": raw,
                    "fields": fields,
                    "sync_token": sync_token,
                    "updated_after": updated_after,
                }
            ),
            headers=remove_none_from_dict({**self._client_wrapper.get_headers(), "x-account-id": account_id}),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(TemplateResult, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 501:
            raise NotImplementedError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def marketing_push_template_update(
        self, id: str, *, request: MarketingCreatePushTemplateRequestDto, account_id: str
    ) -> CreateTemplateResult:
        """
        Parameters:
            - id: str.

            - request: MarketingCreatePushTemplateRequestDto.

            - account_id: str. The account identifier
        """
        _response = await self._client_wrapper.httpx_client.request(
            "PATCH",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"unified/marketing/templates/push/{id}"),
            json=jsonable_encoder(request),
            headers=remove_none_from_dict({**self._client_wrapper.get_headers(), "x-account-id": account_id}),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(CreateTemplateResult, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 501:
            raise NotImplementedError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)
